@@ -1,6 +1,5 @@
 -- Copyright (c) 2019 The DAML Authors. All rights reserved.
 -- SPDX-License-Identifier: Apache-2.0
-{-# LANGUAGE CPP          #-}
 {-# LANGUAGE TypeFamilies #-}
 
 module Development.IDE.Core.FileStore(
@@ -28,16 +27,22 @@ import           Control.Concurrent.STM.TQueue                (writeTQueue)
 import           Control.Exception
 import           Control.Monad.Extra
 import           Control.Monad.IO.Class
+import qualified Data.Binary                                  as B
 import qualified Data.ByteString                              as BS
+import qualified Data.ByteString.Lazy                         as LBS
 import qualified Data.HashMap.Strict                          as HashMap
 import           Data.IORef
+import           Data.List                                    (foldl')
 import qualified Data.Text                                    as T
+import qualified Data.Text                                    as Text
 import qualified Data.Text.Utf16.Rope                         as Rope
 import           Data.Time
 import           Data.Time.Clock.POSIX
 import           Development.IDE.Core.FileUtils
+import           Development.IDE.Core.IdeConfiguration        (isWorkspaceFile)
 import           Development.IDE.Core.RuleTypes
 import           Development.IDE.Core.Shake                   hiding (Log)
+import qualified Development.IDE.Core.Shake                   as Shake
 import           Development.IDE.GHC.Orphans                  ()
 import           Development.IDE.Graph
 import           Development.IDE.Import.DependencyInformation
@@ -45,24 +50,6 @@ import           Development.IDE.Types.Diagnostics
 import           Development.IDE.Types.Location
 import           Development.IDE.Types.Options
 import           HieDb.Create                                 (deleteMissingRealFiles)
-import           Ide.Plugin.Config                            (CheckParents (..),
-                                                               Config)
-import           System.IO.Error
-
-#ifdef mingw32_HOST_OS
-import qualified System.Directory                             as Dir
-#else
-#endif
-
-import qualified Ide.Logger                                   as L
-
-import           Data.Aeson                                   (ToJSON (toJSON))
-import qualified Data.Binary                                  as B
-import qualified Data.ByteString.Lazy                         as LBS
-import           Data.List                                    (foldl')
-import qualified Data.Text                                    as Text
-import           Development.IDE.Core.IdeConfiguration        (isWorkspaceFile)
-import qualified Development.IDE.Core.Shake                   as Shake
 import           Ide.Logger                                   (Pretty (pretty),
                                                                Priority (Info),
                                                                Recorder,
@@ -70,6 +57,9 @@ import           Ide.Logger                                   (Pretty (pretty),
                                                                cmapWithPrio,
                                                                logWith, viaShow,
                                                                (<+>))
+import qualified Ide.Logger                                   as L
+import           Ide.Plugin.Config                            (CheckParents (..),
+                                                               Config)
 import           Language.LSP.Protocol.Message                (toUntypedRegistration)
 import qualified Language.LSP.Protocol.Message                as LSP
 import           Language.LSP.Protocol.Types                  (DidChangeWatchedFilesRegistrationOptions (DidChangeWatchedFilesRegistrationOptions),
@@ -79,8 +69,10 @@ import qualified Language.LSP.Protocol.Types                  as LSP
 import qualified Language.LSP.Server                          as LSP
 import           Language.LSP.VFS
 import           System.FilePath
+import           System.IO.Error
 import           System.IO.Unsafe
 
+
 data Log
   = LogCouldNotIdentifyReverseDeps !NormalizedFilePath
   | LogTypeCheckingReverseDeps !NormalizedFilePath !(Maybe [NormalizedFilePath])
@@ -96,7 +88,7 @@ instance Pretty Log where
       <+> viaShow path
       <> ":"
       <+> pretty (fmap (fmap show) reverseDepPaths)
-    LogShake log -> pretty log
+    LogShake msg -> pretty msg
 
 addWatchedFileRule :: Recorder (WithPriority Log) -> (NormalizedFilePath -> Action Bool) -> Rules ()
 addWatchedFileRule recorder isWatched = defineNoDiagnostics (cmapWithPrio LogShake recorder) $ \AddWatchedFile f -> do
@@ -243,11 +235,10 @@ typecheckParents recorder state nfp = void $ shakeEnqueue (shakeExtras state) pa
 typecheckParentsAction :: Recorder (WithPriority Log) -> NormalizedFilePath -> Action ()
 typecheckParentsAction recorder nfp = do
     revs <- transitiveReverseDependencies nfp <$> useNoFile_ GetModuleGraph
-    let log = logWith recorder
     case revs of
-      Nothing -> log Info $ LogCouldNotIdentifyReverseDeps nfp
+      Nothing -> logWith recorder Info $ LogCouldNotIdentifyReverseDeps nfp
       Just rs -> do
-        log Info $ LogTypeCheckingReverseDeps nfp revs
+        logWith recorder Info $ LogTypeCheckingReverseDeps nfp revs
         void $ uses GetModIface rs
 
 -- | Note that some keys have been modified and restart the session
