@@ -104,7 +104,7 @@ zeroMapping :: PositionMapping
 zeroMapping = PositionMapping idDelta
 
 -- | Compose two position mappings. Composes in the same way as function
--- composition (ie the second argument is applyed to the position first).
+-- composition (ie the second argument is applied to the position first).
 composeDelta :: PositionDelta
                 -> PositionDelta
                 -> PositionDelta
@@ -219,9 +219,9 @@ deltaFromDiff (T.lines -> old) (T.lines -> new) =
           line' -> PositionExact (Position (fromIntegral line') col)
 
     -- Construct a mapping between lines in the diff
-    -- -1 for unsucessful mapping
+    -- -1 for unsuccessful mapping
     go :: [Diff T.Text] -> Int -> Int -> ([Int], [Int])
     go [] _ _ = ([],[])
-    go (Both _ _ : xs) !lold !lnew = bimap  (lnew :) (lold :) $ go xs (lold+1) (lnew+1)
-    go (First _  : xs) !lold !lnew = first  (-1   :)          $ go xs (lold+1) lnew
-    go (Second _ : xs) !lold !lnew = second          (-1   :) $ go xs lold     (lnew+1)
+    go (Both _ _ : xs) !glold !glnew = bimap  (glnew :) (glold :) $ go xs (glold+1) (glnew+1)
+    go (First _  : xs) !glold !glnew = first  (-1   :)          $ go xs (glold+1) glnew
+    go (Second _ : xs) !glold !glnew = second          (-1   :) $ go xs glold     (glnew+1)
