@@ -61,26 +61,25 @@ module Development.IDE.Core.Rules(
     DisplayTHWarning(..),
     ) where
 
+import           Prelude                                      hiding (mod)
 import           Control.Applicative
 import           Control.Concurrent.Async                     (concurrently)
 import           Control.Concurrent.Strict
 import           Control.DeepSeq
 import           Control.Exception.Safe
 import           Control.Exception                            (evaluate)
-import           Control.Monad.Extra
-import           Control.Monad.Reader
-import           Control.Monad.State
+import           Control.Monad.Extra                          hiding (msum)
+import           Control.Monad.Reader                         hiding (msum)
+import           Control.Monad.State                          hiding (msum)
 import           Control.Monad.Trans.Except                   (ExceptT, except,
                                                                runExceptT)
 import           Control.Monad.Trans.Maybe
-import           Data.Aeson                                   (Result (Success),
-                                                               toJSON)
-import qualified Data.Aeson.Types                             as A
+import           Data.Aeson                                   (toJSON)
 import qualified Data.Binary                                  as B
 import qualified Data.ByteString                              as BS
 import qualified Data.ByteString.Lazy                         as LBS
 import           Data.Coerce
-import           Data.Foldable
+import           Data.Foldable                                hiding (msum)
 import qualified Data.HashMap.Strict                          as HM
 import qualified Data.HashSet                                 as HashSet
 import           Data.Hashable
@@ -116,7 +115,7 @@ import           Development.IDE.GHC.Compat                   hiding
                                                                TargetId(..),
                                                                loadInterface,
                                                                Var,
-                                                               (<+>))
+                                                               (<+>), settings)
 import qualified Development.IDE.GHC.Compat                   as Compat hiding (vcat, nest)
 import qualified Development.IDE.GHC.Compat.Util              as Util
 import           Development.IDE.GHC.Error
@@ -146,9 +145,8 @@ import           Ide.Plugin.Properties                        (HasProperty,
                                                                Properties,
                                                                ToHsType,
                                                                useProperty)
-import           Ide.PluginUtils                              (configForPlugin)
 import           Ide.Types                                    (DynFlagsModifications (dynFlagsModifyGlobal, dynFlagsModifyParser),
-                                                               PluginId, PluginDescriptor (pluginId), IdePlugins (IdePlugins))
+                                                               PluginId)
 import Control.Concurrent.STM.Stats (atomically)
 import Language.LSP.Server (LspT)
 import System.Info.Extra (isWindows)
@@ -157,20 +155,24 @@ import Ide.Logger (Recorder, logWith, cmapWithPrio, WithPriority, Pretty (pretty
 import qualified Development.IDE.Core.Shake as Shake
 import qualified Ide.Logger as Logger
 import qualified Development.IDE.Types.Shake as Shake
-import           Development.IDE.GHC.CoreFile
 import           Data.Time.Clock.POSIX             (posixSecondsToUTCTime)
 import Control.Monad.IO.Unlift
-import qualified Data.IntMap as IM
-#if MIN_VERSION_ghc(9,3,0)
-import GHC.Unit.Module.Graph
-import GHC.Unit.Env
-#endif
-#if MIN_VERSION_ghc(9,5,0)
-import GHC.Unit.Home.ModInfo
-#endif
-import GHC (mgModSummaries)
+
+
 import GHC.Fingerprint
 
+-- See Note [Guidelines For Using CPP In GHCIDE Import Statements]
+
+#if !MIN_VERSION_ghc(9,3,0)
+import GHC (mgModSummaries)
+#endif
+
+#if MIN_VERSION_ghc(9,3,0)
+import qualified Data.IntMap as IM
+#endif
+
+
+
 data Log
   = LogShake Shake.Log
   | LogReindexingHieFile !NormalizedFilePath
@@ -182,7 +184,7 @@ data Log
 
 instance Pretty Log where
   pretty = \case
-    LogShake log -> pretty log
+    LogShake msg -> pretty msg
     LogReindexingHieFile path ->
       "Re-indexing hie file for" <+> pretty (fromNormalizedFilePath path)
     LogLoadingHieFile path ->
@@ -334,10 +336,10 @@ getParsedModuleWithCommentsRule recorder =
 
     let ms' = withoutOption Opt_Haddock $ withOption Opt_KeepRawTokenStream ms
     modify_dflags <- getModifyDynFlags dynFlagsModifyParser
-    let ms = ms' { ms_hspp_opts = modify_dflags $ ms_hspp_opts ms' }
+    let ms'' = ms' { ms_hspp_opts = modify_dflags $ ms_hspp_opts ms' }
         reset_ms pm = pm { pm_mod_summary = ms' }
 
-    liftIO $ fmap (fmap reset_ms) $ snd <$> getParsedModuleDefinition hsc opt file ms
+    liftIO $ fmap (fmap reset_ms) $ snd <$> getParsedModuleDefinition hsc opt file ms''
 
 getModifyDynFlags :: (DynFlagsModifications -> a) -> Action a
 getModifyDynFlags f = do
@@ -370,7 +372,7 @@ getLocatedImportsRule recorder =
         let import_dirs = deps env_eq
         let dflags = hsc_dflags env
             isImplicitCradle = isNothing $ envImportPaths env_eq
-        dflags <- return $ if isImplicitCradle
+        dflags' <- return $ if isImplicitCradle
                     then addRelativeImport file (moduleName $ ms_mod ms) dflags
                     else dflags
         opt <- getIdeOptions
@@ -391,7 +393,7 @@ getLocatedImportsRule recorder =
                 | otherwise
                 = return Nothing
         (diags, imports') <- fmap unzip $ forM imports $ \(isSource, (mbPkgName, modName)) -> do
-            diagOrImp <- locateModule (hscSetFlags dflags env) import_dirs (optExtensions opt) getTargetFor modName mbPkgName isSource
+            diagOrImp <- locateModule (hscSetFlags dflags' env) import_dirs (optExtensions opt) getTargetFor modName mbPkgName isSource
             case diagOrImp of
                 Left diags              -> pure (diags, Just (modName, Nothing))
                 Right (FileImport path) -> pure ([], Just (modName, Just path))
@@ -405,7 +407,7 @@ getLocatedImportsRule recorder =
         bootArtifact <- if boot == Just True
               then do
                 let modName = ms_mod_name ms
-                loc <- liftIO $ mkHomeModLocation dflags modName (fromNormalizedFilePath bootPath)
+                loc <- liftIO $ mkHomeModLocation dflags' modName (fromNormalizedFilePath bootPath)
                 return $ Just (noLoc modName, Just (ArtifactsLocation bootPath (Just loc) True))
               else pure Nothing
         -}
@@ -536,9 +538,8 @@ reportImportCyclesRule recorder =
                   let cycles = mapMaybe (cycleErrorInFile fileId) (toList errs)
                   -- Convert cycles of files into cycles of module names
                   forM cycles $ \(imp, files) -> do
-                      modNames <- forM files $ \fileId -> do
-                          let file = idToPath depPathIdMap fileId
-                          getModuleName file
+                      modNames <- forM files $ 
+                          getModuleName . idToPath depPathIdMap
                       pure $ toDiag imp $ sort modNames
     where cycleErrorInFile f (PartOfCycle imp fs)
             | f `elem` fs = Just (imp, fs)
@@ -649,10 +650,9 @@ readHieFileFromDisk :: Recorder (WithPriority Log) -> FilePath -> ExceptT SomeEx
 readHieFileFromDisk recorder hie_loc = do
   nc <- asks ideNc
   res <- liftIO $ tryAny $ loadHieFile (mkUpdater nc) hie_loc
-  let log = (liftIO .) . logWith recorder
   case res of
-    Left e -> log Logger.Debug $ LogLoadingHieFileFail hie_loc e
-    Right _ -> log Logger.Debug $ LogLoadingHieFileSuccess hie_loc
+    Left e -> liftIO $ logWith recorder Logger.Debug $ LogLoadingHieFileFail hie_loc e
+    Right _ -> liftIO $ logWith recorder Logger.Debug $ LogLoadingHieFileSuccess hie_loc
   except res
 
 -- | Typechecks a module.
@@ -852,7 +852,6 @@ getModIfaceFromDiskRule recorder = defineEarlyCutoff (cmapWithPrio LogShake reco
     Just session -> do
       linkableType <- getLinkableType f
       ver <- use_ GetModificationTime f
-      ShakeExtras{ideNc} <- getShakeExtras
       let m_old = case old of
             Shake.Succeeded (Just old_version) v -> Just (v, old_version)
             Shake.Stale _   (Just old_version) v -> Just (v, old_version)
@@ -889,12 +888,12 @@ getModIfaceFromDiskAndIndexRule recorder =
   -- GetModIfaceFromDisk should have written a `.hie` file, must check if it matches version in db
   let ms = hirModSummary x
       hie_loc = Compat.ml_hie_file $ ms_location ms
-  hash <- liftIO $ Util.getFileHash hie_loc
+  fileHash <- liftIO $ Util.getFileHash hie_loc
   mrow <- liftIO $ withHieDb (\hieDb -> HieDb.lookupHieFileFromSource hieDb (fromNormalizedFilePath f))
   hie_loc' <- liftIO $ traverse (makeAbsolute . HieDb.hieModuleHieFile) mrow
   case mrow of
     Just row
-      | hash == HieDb.modInfoHash (HieDb.hieModInfo row)
+      | fileHash == HieDb.modInfoHash (HieDb.hieModInfo row)
       && Just hie_loc == hie_loc'
       -> do
       -- All good, the db has indexed the file
@@ -911,7 +910,7 @@ getModIfaceFromDiskAndIndexRule recorder =
         -- can just re-index the file we read from disk
         Right hf -> liftIO $ do
           logWith recorder Logger.Debug $ LogReindexingHieFile f
-          indexHieFile se ms f hash hf
+          indexHieFile se ms f fileHash hf
 
   return (Just x)
 
@@ -955,8 +954,8 @@ getModSummaryRule displayTHWarning recorder = do
             Left diags -> return (Nothing, (diags, Nothing))
 
     defineEarlyCutoff (cmapWithPrio LogShake recorder) $ RuleNoDiagnostics $ \GetModSummaryWithoutTimestamps f -> do
-        ms <- use GetModSummary f
-        case ms of
+        mbMs <- use GetModSummary f
+        case mbMs of
             Just res@ModSummaryResult{..} -> do
                 let ms = msrModSummary {
 #if !MIN_VERSION_ghc(9,3,0)
@@ -982,7 +981,7 @@ generateCoreRule recorder =
 getModIfaceRule :: Recorder (WithPriority Log) -> Rules ()
 getModIfaceRule recorder = defineEarlyCutoff (cmapWithPrio LogShake recorder) $ Rule $ \GetModIface f -> do
   fileOfInterest <- use_ IsFileOfInterest f
-  res@(_,(_,mhmi)) <- case fileOfInterest of
+  res <- case fileOfInterest of
     IsFOI status -> do
       -- Never load from disk for files of interest
       tmr <- use_ TypeCheck f
@@ -990,14 +989,14 @@ getModIfaceRule recorder = defineEarlyCutoff (cmapWithPrio LogShake recorder) $
       hsc <- hscEnv <$> use_ GhcSessionDeps f
       let compile = fmap ([],) $ use GenerateCore f
       se <- getShakeExtras
-      (diags, !hiFile) <- writeCoreFileIfNeeded se hsc linkableType compile tmr
-      let fp = hiFileFingerPrint <$> hiFile
-      hiDiags <- case hiFile of
+      (diags, !mbHiFile) <- writeCoreFileIfNeeded se hsc linkableType compile tmr
+      let fp = hiFileFingerPrint <$> mbHiFile
+      hiDiags <- case mbHiFile of
         Just hiFile
           | OnDisk <- status
           , not (tmrDeferredError tmr) -> liftIO $ writeHiFile se hsc hiFile
         _ -> pure []
-      return (fp, (diags++hiDiags, hiFile))
+      return (fp, (diags++hiDiags, mbHiFile))
     NotFOI -> do
       hiFile <- use GetModIfaceFromDiskAndIndex f
       let fp = hiFileFingerPrint <$> hiFile
@@ -1029,22 +1028,22 @@ regenerateHiFile sess f ms compNeeded = do
 
     -- Embed haddocks in the interface file
     (diags, mb_pm) <- liftIO $ getParsedModuleDefinition hsc opt f (withOptHaddock ms)
-    (diags, mb_pm) <-
+    (diags', mb_pm') <-
         -- We no longer need to parse again if GHC version is above 9.0. https://github.com/haskell/haskell-language-server/issues/1892
         if Compat.ghcVersion >= Compat.GHC90 || isJust mb_pm then do
             return (diags, mb_pm)
         else do
             -- if parsing fails, try parsing again with Haddock turned off
-            (diagsNoHaddock, mb_pm) <- liftIO $ getParsedModuleDefinition hsc opt f ms
-            return (mergeParseErrorsHaddock diagsNoHaddock diags, mb_pm)
-    case mb_pm of
-        Nothing -> return (diags, Nothing)
+            (diagsNoHaddock, mb_pm') <- liftIO $ getParsedModuleDefinition hsc opt f ms
+            return (mergeParseErrorsHaddock diagsNoHaddock diags, mb_pm')
+    case mb_pm' of
+        Nothing -> return (diags', Nothing)
         Just pm -> do
             -- Invoke typechecking directly to update it without incurring a dependency
             -- on the parsed module and the typecheck rules
-            (diags', mtmr) <- typeCheckRuleDefinition hsc pm
+            (diags'', mtmr) <- typeCheckRuleDefinition hsc pm
             case mtmr of
-              Nothing -> pure (diags', Nothing)
+              Nothing -> pure (diags'', Nothing)
               Just tmr -> do
 
                 let compile = liftIO $ compileModule (RunSimplifier True) hsc (pm_mod_summary pm) $ tmrTypechecked tmr
@@ -1052,7 +1051,7 @@ regenerateHiFile sess f ms compNeeded = do
                 se <- getShakeExtras
 
                 -- Bang pattern is important to avoid leaking 'tmr'
-                (diags'', !res) <- writeCoreFileIfNeeded se hsc compNeeded compile tmr
+                (diags''', !res) <- writeCoreFileIfNeeded se hsc compNeeded compile tmr
 
                 -- Write hi file
                 hiDiags <- case res of
@@ -1061,22 +1060,22 @@ regenerateHiFile sess f ms compNeeded = do
                     -- Write hie file. Do this before writing the .hi file to
                     -- ensure that we always have a up2date .hie file if we have
                     -- a .hi file
-                    se <- getShakeExtras
+                    se' <- getShakeExtras
                     (gDiags, masts) <- liftIO $ generateHieAsts hsc tmr
                     source <- getSourceFileSource f
                     wDiags <- forM masts $ \asts ->
-                      liftIO $ writeAndIndexHieFile hsc se (tmrModSummary tmr) f (tcg_exports $ tmrTypechecked tmr) asts source
+                      liftIO $ writeAndIndexHieFile hsc se' (tmrModSummary tmr) f (tcg_exports $ tmrTypechecked tmr) asts source
 
                     -- We don't write the `.hi` file if there are deferred errors, since we won't get
                     -- accurate diagnostics next time if we do
                     hiDiags <- if not $ tmrDeferredError tmr
-                               then liftIO $ writeHiFile se hsc hiFile
+                               then liftIO $ writeHiFile se' hsc hiFile
                                else pure []
 
                     pure (hiDiags <> gDiags <> concat wDiags)
                   Nothing -> pure []
 
-                return (diags <> diags' <> diags'' <> hiDiags, res)
+                return (diags' <> diags'' <> diags''' <> hiDiags, res)
 
 
 -- | HscEnv should have deps included already
@@ -1096,6 +1095,7 @@ writeCoreFileIfNeeded se hsc (Just _) getGuts tmr = do
       (diags', !res) <- liftIO $ mkHiFileResultCompile se hsc tmr guts
       pure (diags++diags', res)
 
+-- See Note [Client configuration in Rules]
 getClientSettingsRule :: Recorder (WithPriority Log) -> Rules ()
 getClientSettingsRule recorder = defineEarlyCutOffNoFile (cmapWithPrio LogShake recorder) $ \GetClientSettings -> do
   alwaysRerun
@@ -1126,10 +1126,8 @@ getLinkableRule recorder =
     core_t <- liftIO $ getModTime core_file
     case hirCoreFp of
       Nothing -> error "called GetLinkable for a file without a linkable"
-      Just (bin_core, hash) -> do
+      Just (bin_core, fileHash) -> do
         session <- use_ GhcSessionDeps f
-        ShakeExtras{ideNc} <- getShakeExtras
-        let namecache_updater = mkUpdater ideNc
         linkableType <- getLinkableType f >>= \case
           Nothing -> error "called GetLinkable for a file which doesn't need compilation"
           Just t -> pure t
@@ -1165,9 +1163,9 @@ getLinkableRule recorder =
               --just before returning it to be loaded. This has a substantial effect on recompile
               --times as the number of loaded modules and splices increases.
               --
-              unload (hscEnv session) (map (\(mod, time) -> LM time mod []) $ moduleEnvToList to_keep)
+              unload (hscEnv session) (map (\(mod', time') -> LM time' mod' []) $ moduleEnvToList to_keep)
               return (to_keep, ())
-        return (hash <$ hmi, (warns, LinkableResult <$> hmi <*> pure hash))
+        return (fileHash <$ hmi, (warns, LinkableResult <$> hmi <*> pure fileHash))
 
 -- | For now we always use bytecode unless something uses unboxed sums and tuples along with TH
 getLinkableType :: NormalizedFilePath -> Action (Maybe LinkableType)
@@ -1224,11 +1222,11 @@ computeLinkableTypeForDynFlags d
 #if defined(GHC_PATCHED_UNBOXED_BYTECODE) || MIN_VERSION_ghc(9,2,0)
           = BCOLinkable
 #else
-          | unboxed_tuples_or_sums = ObjectLinkable
+          | _unboxed_tuples_or_sums = ObjectLinkable
           | otherwise              = BCOLinkable
 #endif
-  where
-        unboxed_tuples_or_sums =
+  where -- unboxed_tuples_or_sums is only used in GHC < 9.2
+        _unboxed_tuples_or_sums =
             xopt LangExt.UnboxedTuples d || xopt LangExt.UnboxedSums d
 
 -- | Tracks which linkables are current, so we don't need to unload them
