@@ -53,41 +53,19 @@ module Development.IDE.GHC.Compat.Units (
     findImportedModule,
     ) where
 
-import           Control.Monad
-import qualified Data.List.NonEmpty                    as NE
-import qualified Data.Map.Strict                       as Map
-#if MIN_VERSION_ghc(9,3,0)
-import           GHC.Unit.Home.ModInfo
-#endif
-#if MIN_VERSION_ghc(9,0,0)
-#if MIN_VERSION_ghc(9,2,0)
-import qualified GHC.Data.ShortText                    as ST
-#if !MIN_VERSION_ghc(9,3,0)
-import           GHC.Driver.Env                        (hsc_unit_dbs)
-#endif
-import           GHC.Driver.Ppr
-import           GHC.Unit.Env
-import           GHC.Unit.External
-import           GHC.Unit.Finder                       hiding
-                                                       (findImportedModule)
-#else
-import           GHC.Driver.Types
-#endif
-import           GHC.Data.FastString
-import qualified GHC.Driver.Session                    as DynFlags
-import           GHC.Types.Unique.Set
-import qualified GHC.Unit.Info                         as UnitInfo
-import           GHC.Unit.State                        (LookupResult, UnitInfo,
-                                                        UnitState (unitInfoMap))
-import qualified GHC.Unit.State                        as State
-import           GHC.Unit.Types                        hiding (moduleUnit,
-                                                        toUnitId)
-import qualified GHC.Unit.Types                        as Unit
-import           GHC.Utils.Outputable
-#else
+import           Data.Either
+import           Data.Version
+import           Development.IDE.GHC.Compat.Core
+import           Development.IDE.GHC.Compat.Env
+import           Development.IDE.GHC.Compat.Outputable
+import           Prelude                               hiding (mod)
+
+-- See Note [Guidelines For Using CPP In GHCIDE Import Statements]
+
+#if !MIN_VERSION_ghc(9,0,0)
 import qualified DynFlags
 import           FastString
-import           GhcPlugins                            (SDoc, showSDocForUser)
+import qualified Finder                                as GHC
 import           HscTypes
 import           Module                                hiding (moduleUnitId)
 import qualified Module
@@ -101,26 +79,51 @@ import           Packages                              (InstalledPackageInfo (ha
 import qualified Packages
 #endif
 
-import           Development.IDE.GHC.Compat.Core
-import           Development.IDE.GHC.Compat.Env
-import           Development.IDE.GHC.Compat.Outputable
+#if MIN_VERSION_ghc(9,0,0)
+import           GHC.Types.Unique.Set
+import qualified GHC.Unit.Info                         as UnitInfo
+import           GHC.Unit.State                        (LookupResult, UnitInfo,
+                                                        UnitState (unitInfoMap))
+import qualified GHC.Unit.State                        as State
+import           GHC.Unit.Types                        hiding (moduleUnit,
+                                                        toUnitId)
+import qualified GHC.Unit.Types                        as Unit
+#endif
+
 #if MIN_VERSION_ghc(9,0,0) && !MIN_VERSION_ghc(9,2,0)
 import           Data.Map                              (Map)
+import qualified GHC.Driver.Finder                     as GHC
+import qualified GHC.Driver.Session                    as DynFlags
+import           GHC.Driver.Types
+#endif
+
+#if MIN_VERSION_ghc(9,0,0) && !MIN_VERSION_ghc(9,3,0)
+import           GHC.Data.FastString
+
+#endif
+
+#if MIN_VERSION_ghc(9,2,0)
+import qualified GHC.Data.ShortText                    as ST
+import           GHC.Unit.External
+import qualified GHC.Unit.Finder                       as GHC
+#endif
+
+#if MIN_VERSION_ghc(9,2,0) && !MIN_VERSION_ghc(9,3,0)
+import           GHC.Unit.Env
+import           GHC.Unit.Finder                       hiding
+                                                       (findImportedModule)
 #endif
-import           Data.Either
-import           Data.Version
-import qualified GHC
 
 #if MIN_VERSION_ghc(9,3,0)
+import           Control.Monad
+import qualified Data.List.NonEmpty                    as NE
+import qualified Data.Map.Strict                       as Map
+import qualified GHC
+import qualified GHC.Driver.Session                    as DynFlags
 import           GHC.Types.PkgQual                     (PkgQual (NoPkgQual))
+import           GHC.Unit.Home.ModInfo
 #endif
-#if MIN_VERSION_ghc(9,1,0)
-import qualified GHC.Unit.Finder                       as GHC
-#elif MIN_VERSION_ghc(9,0,0)
-import qualified GHC.Driver.Finder                     as GHC
-#else
-import qualified Finder                                as GHC
-#endif
+
 
 #if MIN_VERSION_ghc(9,0,0)
 type PreloadUnitClosure = UniqSet UnitId
