@@ -1,6 +1,5 @@
-{-# LANGUAGE PackageImports #-}
 {-# OPTIONS_GHC -Wno-orphans #-}
-{-# LANGUAGE RankNTypes     #-}
+{-# LANGUAGE RankNTypes #-}
 module Development.IDE.Main
 (Arguments(..)
 ,defaultArguments
@@ -12,13 +11,18 @@ module Development.IDE.Main
 ,testing
 ,Log(..)
 ) where
+
 import           Control.Concurrent.Extra                 (withNumCapabilities)
+import           Control.Concurrent.MVar                  (newEmptyMVar,
+                                                           putMVar, tryReadMVar)
 import           Control.Concurrent.STM.Stats             (dumpSTMStats)
 import           Control.Exception.Safe                   (SomeException,
                                                            catchAny,
                                                            displayException)
 import           Control.Monad.Extra                      (concatMapM, unless,
                                                            when)
+import           Control.Monad.IO.Class                   (liftIO)
+import qualified Data.Aeson                               as J
 import           Data.Coerce                              (coerce)
 import           Data.Default                             (Default (def))
 import           Data.Foldable                            (traverse_)
@@ -30,14 +34,15 @@ import           Data.List.Extra                          (intercalate,
 import           Data.Maybe                               (catMaybes, isJust)
 import qualified Data.Text                                as T
 import           Development.IDE                          (Action,
-                                                           GhcVersion (..),
                                                            Priority (Debug, Error),
-                                                           Rules, ghcVersion,
+                                                           Rules, emptyFilePath,
                                                            hDuplicateTo')
 import           Development.IDE.Core.Debouncer           (Debouncer,
                                                            newAsyncDebouncer)
-import           Development.IDE.Core.FileStore           (isWatchSupported)
+import           Development.IDE.Core.FileStore           (isWatchSupported,
+                                                           setSomethingModified)
 import           Development.IDE.Core.IdeConfiguration    (IdeConfiguration (..),
+                                                           modifyClientSettings,
                                                            registerIdeConfiguration)
 import           Development.IDE.Core.OfInterest          (FileOfInterestStatus (OnDisk),
                                                            kick,
@@ -77,14 +82,6 @@ import           Development.IDE.Session                  (SessionLoadingOptions
 import qualified Development.IDE.Session                  as Session
 import           Development.IDE.Types.Location           (NormalizedUri,
                                                            toNormalizedFilePath')
-import           Ide.Logger             (Logger,
-                                                           Pretty (pretty),
-                                                           Priority (Info, Warning),
-                                                           Recorder,
-                                                           WithPriority,
-                                                           cmapWithPrio,
-                                                           logWith, nest, vsep,
-                                                           (<+>))
 import           Development.IDE.Types.Monitoring         (Monitoring)
 import           Development.IDE.Types.Options            (IdeGhcSession,
                                                            IdeOptions (optCheckParents, optCheckProject, optReportProgress, optRunSubset),
@@ -93,12 +90,20 @@ import           Development.IDE.Types.Options            (IdeGhcSession,
                                                            defaultIdeOptions,
                                                            optModifyDynFlags,
                                                            optTesting)
-import           Development.IDE.Types.Shake              (WithHieDb)
+import           Development.IDE.Types.Shake              (WithHieDb, toKey)
 import           GHC.Conc                                 (getNumProcessors)
 import           GHC.IO.Encoding                          (setLocaleEncoding)
 import           GHC.IO.Handle                            (hDuplicate)
 import           HIE.Bios.Cradle                          (findCradle)
 import qualified HieDb.Run                                as HieDb
+import           Ide.Logger                               (Logger,
+                                                           Pretty (pretty),
+                                                           Priority (Info),
+                                                           Recorder,
+                                                           WithPriority,
+                                                           cmapWithPrio,
+                                                           logDebug, logWith,
+                                                           nest, vsep, (<+>))
 import           Ide.Plugin.Config                        (CheckParents (NeverCheck),
                                                            Config, checkParents,
                                                            checkProject,
@@ -147,7 +152,7 @@ data Log
 
 instance Pretty Log where
   pretty = \case
-    LogHeapStats log -> pretty log
+    LogHeapStats msg -> pretty msg
     LogLspStart pluginIds ->
       nest 2 $ vsep
         [ "Starting LSP server..."
@@ -160,13 +165,13 @@ instance Pretty Log where
       "shouldRunSubset:" <+> pretty shouldRunSubset
     LogSetInitialDynFlagsException e ->
       "setInitialDynFlags:" <+> pretty (displayException e)
-    LogService log -> pretty log
-    LogShake log -> pretty log
-    LogGhcIde log -> pretty log
-    LogLanguageServer log -> pretty log
-    LogSession log -> pretty log
-    LogPluginHLS log -> pretty log
-    LogRules log -> pretty log
+    LogService msg -> pretty msg
+    LogShake msg -> pretty msg
+    LogGhcIde msg -> pretty msg
+    LogLanguageServer msg -> pretty msg
+    LogSession msg -> pretty msg
+    LogPluginHLS msg -> pretty msg
+    LogRules msg -> pretty msg
 
 data Command
     = Check [FilePath]  -- ^ Typecheck some paths and print diagnostics. Exit code is the number of failures
@@ -281,9 +286,6 @@ testing recorder logger plugins =
 defaultMain :: Recorder (WithPriority Log) -> Arguments -> IO ()
 defaultMain recorder Arguments{..} = withHeapStats (cmapWithPrio LogHeapStats recorder) fun
  where
-  log :: Priority -> Log -> IO ()
-  log = logWith recorder
-
   fun = do
     setLocaleEncoding utf8
     pid <- T.pack . show <$> getProcessID
@@ -294,7 +296,7 @@ defaultMain recorder Arguments{..} = withHeapStats (cmapWithPrio LogHeapStats re
         hlsCommands = allLspCmdIds' pid argsHlsPlugins
         plugins = hlsPlugin <> argsGhcidePlugin
         options = argsLspOptions { LSP.optExecuteCommandCommands = LSP.optExecuteCommandCommands argsLspOptions <> Just hlsCommands }
-        argsOnConfigChange = getConfigFromNotification argsHlsPlugins
+        argsParseConfig = getConfigFromNotification argsHlsPlugins
         rules = argsRules >> pluginRules plugins
 
     debouncer <- argsDebouncer
@@ -306,14 +308,15 @@ defaultMain recorder Arguments{..} = withHeapStats (cmapWithPrio LogHeapStats re
 
     case argCommand of
         LSP -> withNumCapabilities numCapabilities $ do
-            t <- offsetTime
-            log Info $ LogLspStart (pluginId <$> ipMap argsHlsPlugins)
+            ioT <- offsetTime
+            logWith recorder Info $ LogLspStart (pluginId <$> ipMap argsHlsPlugins)
 
+            ideStateVar <- newEmptyMVar
             let getIdeState :: LSP.LanguageContextEnv Config -> Maybe FilePath -> WithHieDb -> IndexQueue -> IO IdeState
                 getIdeState env rootPath withHieDb hieChan = do
                   traverse_ IO.setCurrentDirectory rootPath
-                  t <- t
-                  log Info $ LogLspStartDuration t
+                  t <- ioT
+                  logWith recorder Info $ LogLspStartDuration t
 
                   dir <- maybe IO.getCurrentDirectory return rootPath
 
@@ -322,7 +325,7 @@ defaultMain recorder Arguments{..} = withHeapStats (cmapWithPrio LogHeapStats re
                   _mlibdir <-
                       setInitialDynFlags (cmapWithPrio LogSession recorder) dir argsSessionLoadingOptions
                           -- TODO: should probably catch/log/rethrow at top level instead
-                          `catchAny` (\e -> log Error (LogSetInitialDynFlagsException e) >> pure Nothing)
+                          `catchAny` (\e -> logWith recorder Error (LogSetInitialDynFlagsException e) >> pure Nothing)
 
                   sessionLoader <- loadSessionWithOptions (cmapWithPrio LogSession recorder) argsSessionLoadingOptions dir
                   config <- LSP.runLspT env LSP.getConfig
@@ -330,16 +333,16 @@ defaultMain recorder Arguments{..} = withHeapStats (cmapWithPrio LogHeapStats re
 
                   -- disable runSubset if the client doesn't support watched files
                   runSubset <- (optRunSubset def_options &&) <$> LSP.runLspT env isWatchSupported
-                  log Debug $ LogShouldRunSubset runSubset
+                  logWith recorder Debug $ LogShouldRunSubset runSubset
 
-                  let options = def_options
+                  let ideOptions = def_options
                               { optReportProgress = clientSupportsProgress caps
                               , optModifyDynFlags = optModifyDynFlags def_options <> pluginModifyDynflags plugins
                               , optRunSubset = runSubset
                               }
                       caps = LSP.resClientCapabilities env
                   monitoring <- argsMonitoring
-                  initialise
+                  ide <- initialise
                       (cmapWithPrio LogService recorder)
                       argsDefaultHlsConfig
                       argsHlsPlugins
@@ -347,14 +350,28 @@ defaultMain recorder Arguments{..} = withHeapStats (cmapWithPrio LogHeapStats re
                       (Just env)
                       logger
                       debouncer
-                      options
+                      ideOptions
                       withHieDb
                       hieChan
                       monitoring
+                  putMVar ideStateVar ide
+                  pure ide
 
             let setup = setupLSP (cmapWithPrio LogLanguageServer recorder) argsGetHieDbLoc (pluginHandlers plugins) getIdeState
+                -- See Note [Client configuration in Rules]
+                onConfigChange cfg = do
+                  -- TODO: this is nuts, we're converting back to JSON just to get a fingerprint
+                  let cfgObj = J.toJSON cfg
+                  mide <- liftIO $ tryReadMVar ideStateVar
+                  case mide of
+                    Nothing -> pure ()
+                    Just ide -> liftIO $ do
+                        let msg = T.pack $ show cfg
+                        logDebug (Shake.ideLogger ide) $ "Configuration changed: " <> msg
+                        modifyClientSettings ide (const $ Just cfgObj)
+                        setSomethingModified Shake.VFSUnmodified ide [toKey Rules.GetClientSettings emptyFilePath] "config change"
 
-            runLanguageServer (cmapWithPrio LogLanguageServer recorder) options inH outH argsDefaultHlsConfig argsOnConfigChange setup
+            runLanguageServer (cmapWithPrio LogLanguageServer recorder) options inH outH argsDefaultHlsConfig argsParseConfig onConfigChange setup
             dumpSTMStats
         Check argFiles -> do
           dir <- maybe IO.getCurrentDirectory return argsProjectRoot
@@ -370,11 +387,11 @@ defaultMain recorder Arguments{..} = withHeapStats (cmapWithPrio LogHeapStats re
             putStrLn $ "\nStep 1/4: Finding files to test in " ++ dir
             files <- expandFiles (argFiles ++ ["." | null argFiles])
             -- LSP works with absolute file paths, so try and behave similarly
-            files <- nubOrd <$> mapM IO.canonicalizePath files
-            putStrLn $ "Found " ++ show (length files) ++ " files"
+            absoluteFiles <- nubOrd <$> mapM IO.canonicalizePath files
+            putStrLn $ "Found " ++ show (length absoluteFiles) ++ " files"
 
             putStrLn "\nStep 2/4: Looking for hie.yaml files that control setup"
-            cradles <- mapM findCradle files
+            cradles <- mapM findCradle absoluteFiles
             let ucradles = nubOrd cradles
             let n = length ucradles
             putStrLn $ "Found " ++ show n ++ " cradle" ++ ['s' | n /= 1]
@@ -382,25 +399,25 @@ defaultMain recorder Arguments{..} = withHeapStats (cmapWithPrio LogHeapStats re
             putStrLn "\nStep 3/4: Initializing the IDE"
             sessionLoader <- loadSessionWithOptions (cmapWithPrio LogSession recorder) argsSessionLoadingOptions dir
             let def_options = argsIdeOptions argsDefaultHlsConfig sessionLoader
-                options = def_options
+                ideOptions = def_options
                         { optCheckParents = pure NeverCheck
                         , optCheckProject = pure False
                         , optModifyDynFlags = optModifyDynFlags def_options <> pluginModifyDynflags plugins
                         }
-            ide <- initialise (cmapWithPrio LogService recorder) argsDefaultHlsConfig argsHlsPlugins rules Nothing logger debouncer options hiedb hieChan mempty
+            ide <- initialise (cmapWithPrio LogService recorder) argsDefaultHlsConfig argsHlsPlugins rules Nothing logger debouncer ideOptions hiedb hieChan mempty
             shakeSessionInit (cmapWithPrio LogShake recorder) ide
             registerIdeConfiguration (shakeExtras ide) $ IdeConfiguration mempty (hashed Nothing)
 
             putStrLn "\nStep 4/4: Type checking the files"
-            setFilesOfInterest ide $ HashMap.fromList $ map ((,OnDisk) . toNormalizedFilePath') files
-            results <- runAction "User TypeCheck" ide $ uses TypeCheck (map toNormalizedFilePath' files)
-            _results <- runAction "GetHie" ide $ uses GetHieAst (map toNormalizedFilePath' files)
-            _results <- runAction "GenerateCore" ide $ uses GenerateCore (map toNormalizedFilePath' files)
-            let (worked, failed) = partition fst $ zip (map isJust results) files
+            setFilesOfInterest ide $ HashMap.fromList $ map ((,OnDisk) . toNormalizedFilePath') absoluteFiles
+            results <- runAction "User TypeCheck" ide $ uses TypeCheck (map toNormalizedFilePath' absoluteFiles)
+            _results <- runAction "GetHie" ide $ uses GetHieAst (map toNormalizedFilePath' absoluteFiles)
+            _results <- runAction "GenerateCore" ide $ uses GenerateCore (map toNormalizedFilePath' absoluteFiles)
+            let (worked, failed) = partition fst $ zip (map isJust results) absoluteFiles
             when (failed /= []) $
                 putStr $ unlines $ "Files that failed:" : map ((++) " * " . snd) failed
 
-            let nfiles xs = let n = length xs in if n == 1 then "1 file" else show n ++ " files"
+            let nfiles xs = let n' = length xs in if n' == 1 then "1 file" else show n' ++ " files"
             putStrLn $ "\nCompleted (" ++ nfiles worked ++ " worked, " ++ nfiles failed ++ " failed)"
 
             unless (null failed) (exitWith $ ExitFailure (length failed))
@@ -420,12 +437,12 @@ defaultMain recorder Arguments{..} = withHeapStats (cmapWithPrio LogHeapStats re
           runWithDb (cmapWithPrio LogSession recorder) dbLoc $ \hiedb hieChan -> do
             sessionLoader <- loadSessionWithOptions (cmapWithPrio LogSession recorder) argsSessionLoadingOptions "."
             let def_options = argsIdeOptions argsDefaultHlsConfig sessionLoader
-                options = def_options
+                ideOptions = def_options
                     { optCheckParents = pure NeverCheck
                     , optCheckProject = pure False
                     , optModifyDynFlags = optModifyDynFlags def_options <> pluginModifyDynflags plugins
                     }
-            ide <- initialise (cmapWithPrio LogService recorder) argsDefaultHlsConfig argsHlsPlugins rules Nothing logger debouncer options hiedb hieChan mempty
+            ide <- initialise (cmapWithPrio LogService recorder) argsDefaultHlsConfig argsHlsPlugins rules Nothing logger debouncer ideOptions hiedb hieChan mempty
             shakeSessionInit (cmapWithPrio LogShake recorder) ide
             registerIdeConfiguration (shakeExtras ide) $ IdeConfiguration mempty (hashed Nothing)
             c ide
@@ -437,9 +454,9 @@ expandFiles = concatMapM $ \x -> do
         then return [x]
         else do
             let recurse "." = True
-                recurse x | "." `isPrefixOf` takeFileName x = False -- skip .git etc
-                recurse x = takeFileName x `notElem` ["dist", "dist-newstyle"] -- cabal directories
-            files <- filter (\x -> takeExtension x `elem` [".hs", ".lhs"]) <$> IO.listFilesInside (return . recurse) x
+                recurse y | "." `isPrefixOf` takeFileName y = False -- skip .git etc
+                recurse y = takeFileName y `notElem` ["dist", "dist-newstyle"] -- cabal directories
+            files <- filter (\y -> takeExtension y `elem` [".hs", ".lhs"]) <$> IO.listFilesInside (return . recurse) x
             when (null files) $
                 fail $ "Couldn't find any .hs/.lhs files inside directory: " ++ x
             return files
