@@ -11,7 +11,6 @@ module Development.IDE.Plugin.HLS
     ) where
 
 import           Control.Exception             (SomeException)
-import           Control.Lens                  ((^.))
 import           Control.Monad
 import           Control.Monad.Trans.Except    (runExceptT)
 import qualified Data.Aeson                    as A
@@ -39,7 +38,6 @@ import           Ide.Plugin.Config
 import           Ide.Plugin.Error
 import           Ide.PluginUtils               (getClientConfig)
 import           Ide.Types                     as HLS
-import qualified Language.LSP.Protocol.Lens    as L
 import           Language.LSP.Protocol.Message
 import           Language.LSP.Protocol.Types
 import qualified Language.LSP.Server           as LSP
@@ -65,22 +63,15 @@ instance Pretty Log where
     LogPluginError (PluginId pId) err ->
       pretty pId <> ":" <+> pretty err
     LogResponseError (PluginId pId) err ->
-      pretty pId <> ":" <+> prettyResponseError err
+      pretty pId <> ":" <+> pretty err
     LogNoPluginForMethod (Some method) ->
-        "No plugin enabled for " <> pretty (show method)
+        "No plugin enabled for " <> pretty method
     LogInvalidCommandIdentifier-> "Invalid command identifier"
     ExceptionInPlugin plId (Some method) exception ->
         "Exception in plugin " <> viaShow plId <> " while processing "
-          <> viaShow method <> ": " <> viaShow exception
+          <> pretty method <> ": " <> viaShow exception
 instance Show Log where show = renderString . layoutCompact . pretty
 
--- various error message specific builders
-prettyResponseError :: ResponseError -> Doc a
-prettyResponseError err = errorCode <> ":" <+> errorBody
-    where
-        errorCode = pretty $ show $ err ^. L.code
-        errorBody = pretty $ err ^. L.message
-
 noPluginEnabled :: Recorder (WithPriority Log) -> SMethod m -> [PluginId] -> IO (Either ResponseError c)
 noPluginEnabled recorder m fs' = do
   logWith recorder Warning (LogNoPluginForMethod $ Some m)
@@ -91,7 +82,7 @@ noPluginEnabled recorder m fs' = do
         pluginNotEnabled method availPlugins =
             "No plugin enabled for " <> T.pack (show method) <> ", potentially available: "
                 <> (T.intercalate ", " $ map (\(PluginId plid) -> plid) availPlugins)
-  
+
 pluginDoesntExist :: PluginId -> Text
 pluginDoesntExist (PluginId pid) = "Plugin " <> pid <> " doesn't exist"
 
@@ -232,9 +223,9 @@ executeCommandHandlers recorder ecs = requestHandler SMethod_WorkspaceExecuteCom
 -- ---------------------------------------------------------------------
 
 extensiblePlugins ::  Recorder (WithPriority Log) -> [(PluginId, PluginDescriptor IdeState)] -> Plugin Config
-extensiblePlugins recorder xs = mempty { P.pluginHandlers = handlers }
+extensiblePlugins recorder plugins = mempty { P.pluginHandlers = handlers }
   where
-    IdeHandlers handlers' = foldMap bakePluginId xs
+    IdeHandlers handlers' = foldMap bakePluginId plugins
     bakePluginId :: (PluginId, PluginDescriptor IdeState) -> IdeHandlers
     bakePluginId (pid,pluginDesc) = IdeHandlers $ DMap.map
       (\(PluginHandler f) -> IdeHandler [(pid,pluginDesc,f pid)])
@@ -250,11 +241,11 @@ extensiblePlugins recorder xs = mempty { P.pluginHandlers = handlers }
         -- Clients generally don't display ResponseErrors so instead we log any that we come across
         case nonEmpty fs of
           Nothing -> liftIO $ noPluginEnabled recorder m ((\(x, _, _) -> x) <$> fs')
-          Just fs -> do
-            let  handlers = fmap (\(plid,_,handler) -> (plid,handler)) fs
-            es <- runConcurrently exceptionInPlugin m handlers ide params
+          Just neFs -> do
+            let  plidsAndHandlers = fmap (\(plid,_,handler) -> (plid,handler)) neFs
+            es <- runConcurrently exceptionInPlugin m plidsAndHandlers ide params
             caps <- LSP.getClientCapabilities
-            let (errs,succs) = partitionEithers $ toList $ join $ NE.zipWith (\(pId,_) -> fmap (first (pId,))) handlers es
+            let (errs,succs) = partitionEithers $ toList $ join $ NE.zipWith (\(pId,_) -> fmap (first (pId,))) plidsAndHandlers es
             liftIO $ unless (null errs) $ logErrors recorder errs
             case nonEmpty succs of
               Nothing -> do
@@ -288,12 +279,12 @@ extensibleNotificationPlugins recorder xs = mempty { P.pluginHandlers = handlers
         case nonEmpty fs of
           Nothing -> do
             logWith recorder Warning (LogNoPluginForMethod $ Some m)
-          Just fs -> do
+          Just neFs -> do
             -- We run the notifications in order, so the core ghcide provider
             -- (which restarts the shake process) hopefully comes last
             mapM_ (\(pid,_,f) -> otTracedProvider pid (fromString $ show m) $ f ide vfs params
                                     `catchAny` -- See Note [Exception handling in plugins]
-                                    (\e -> logWith recorder Warning (ExceptionInPlugin pid (Some m) e))) fs
+                                    (\e -> logWith recorder Warning (ExceptionInPlugin pid (Some m) e))) neFs
 
 
 -- ---------------------------------------------------------------------
@@ -344,14 +335,14 @@ newtype IdeNotificationHandlers = IdeNotificationHandlers (DMap IdeNotification
 instance Semigroup IdeHandlers where
   (IdeHandlers a) <> (IdeHandlers b) = IdeHandlers $ DMap.unionWithKey go a b
     where
-      go _ (IdeHandler a) (IdeHandler b) = IdeHandler (a <> b)
+      go _ (IdeHandler c) (IdeHandler d) = IdeHandler (c <> d)
 instance Monoid IdeHandlers where
   mempty = IdeHandlers mempty
 
 instance Semigroup IdeNotificationHandlers where
   (IdeNotificationHandlers a) <> (IdeNotificationHandlers b) = IdeNotificationHandlers $ DMap.unionWithKey go a b
     where
-      go _ (IdeNotificationHandler a) (IdeNotificationHandler b) = IdeNotificationHandler (a <> b)
+      go _ (IdeNotificationHandler c) (IdeNotificationHandler d) = IdeNotificationHandler (c <> d)
 instance Monoid IdeNotificationHandlers where
   mempty = IdeNotificationHandlers mempty
 
